"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
var fs_1 = require("fs");
var path_1 = require("path");
var regex_parser_1 = __importDefault(require("regex-parser"));
var escape = require("escape-string-regexp");
var DirectorySearcher = /** @class */ (function () {
    function DirectorySearcher(options) {
        this.options = options;
    }
    Object.defineProperty(DirectorySearcher.prototype, "fullBase", {
        get: function () {
            return path_1.resolve(this.options.basedir);
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(DirectorySearcher.prototype, "isInBase", {
        get: function () {
            return regex_parser_1.default("/^" + escape(this.fullBase) + "/");
        },
        enumerable: true,
        configurable: true
    });
    DirectorySearcher.prototype.search = function (directory) {
        var _this = this;
        if (directory === void 0) { directory = this.options.basedir; }
        var fullBase = this.fullBase; //resolve(this.options.basedir);
        var isInBase = this.isInBase; //parser(`/^${escape(fullBase)}/`);
        var dir = fs_1.readdirSync(directory);
        var directoryMap = dir.map(function (file) {
            switch (file) {
                case ".":
                    return null;
                case "..":
                    return null;
                default:
                    var filepath = path_1.join(directory, file);
                    var stat = fs_1.statSync(filepath);
                    var ext = path_1.extname(filepath);
                    var relativePath = _this.getRelativeRoutePath(filepath, ext);
                    var result = {
                        directory: directory,
                        filepath: filepath,
                        stat: stat,
                        isFile: stat.isFile(),
                        isDirectory: stat.isDirectory(),
                        ext: ext,
                        relativePath: relativePath,
                    };
                    return result;
            }
        }).filter(function (x) { return x; }).filter(function (x) {
            var fullFile = path_1.resolve(x.filepath);
            return isInBase.test(fullFile);
        });
        var directoriesOnly = directoryMap.filter(function (x) {
            return x.isDirectory;
        });
        var filesOnly = directoryMap.filter(function (x) {
            return x.isFile;
        }).filter(function (x) {
            return _this.options.ext.map(function (x) { return x.toLocaleLowerCase(); }).includes(x.ext.toLowerCase());
        });
        var subdirFiles = directoriesOnly.map(function (x) { return _this.search(x.filepath); }).reduce(function (a, b) {
            return a.concat(b);
        }, []);
        var files = filesOnly.concat(subdirFiles);
        return files;
    };
    DirectorySearcher.prototype.getRelativeRoutePath = function (filepath, ext) {
        return path_1.resolve(filepath).replace(this.isInBase, "").replace(ext, "").replace(/\[([^\]]+)\]/g, ":$1").replace(/\\{1,2}/g, "/");
    };
    return DirectorySearcher;
}());
exports.DirectorySearcher = DirectorySearcher;
