"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
var DirectorySearcher_1 = require("../DirectorySearcher/DirectorySearcher");
var ExpressRouteBuilder_1 = require("../RouteBuilder/ExpressRouteBuilder");
var express_1 = __importDefault(require("express"));
var ExpressBootstrap = /** @class */ (function () {
    function ExpressBootstrap(app) {
        this.app = app;
    }
    ExpressBootstrap.prototype.boot = function (options) {
        var _this = this;
        var directoryResults = new DirectorySearcher_1.DirectorySearcher(options).search();
        var routes = new ExpressRouteBuilder_1.ExpressRouteBuilder().getRoutes(directoryResults);
        var handles = new ExpressRouteBuilder_1.ExpressHandlerBuilder().build(routes);
        handles.forEach(function (route) {
            var router = express_1.default.Router();
            _this.app.use(route.route, router);
            route.config.forEach(function (handle) {
                router.all(handle.path, function (req, res, next) {
                    var handler = require(handle.required);
                    var name = Object.keys(handler)[0];
                    if (handler[name]) {
                        if (options.verbose) {
                            console.log("\n" + ("\n" + route.route + "\n    " + handle.path + "\n        (" + name + "): " + handle.filepath + "\n                        ").trim() + "\n");
                        }
                        handler[name](req, res, next);
                    }
                    else {
                        next();
                    }
                    if (options.live) {
                        delete require.cache[require.resolve(handle.required)];
                    }
                });
            });
        });
    };
    return ExpressBootstrap;
}());
exports.ExpressBootstrap = ExpressBootstrap;
